import { EntityInventoryComponent, CompoundBlockVolume, CompoundBlockVolumeAction, BlockVolume } from "@minecraft/server";
export function openDoor(block, allowedToOpenSpecialDoor) {
    const perm = block.permutation;
    if (allowedToOpenSpecialDoor === false && !block.hasTag('door:common')) {
        return;
    }
    block.setPermutation(perm.withState('id:door', 2).withState('is:open', false));
}
export function closeDoor(block, allowedToOpenSpecialDoor) {
    const perm = block.permutation;
    if (allowedToOpenSpecialDoor === false && !block.hasTag('door:common')) {
        return;
    }
    if (block.hasTag('hcz_door')) {
        block.setPermutation(perm.withState('id:door', 14).withState('is:open', true));
        return;
    }
    block.setPermutation(perm.withState('id:door', 15).withState('is:open', true));
}
function interactWithDoor(block, allowedToOpenSpecialDoor) {
    const perm = block.permutation;
    const doorState = perm.getState('id:door');
    const upperBlock = block.above(1);
    if (doorState === 1) {
        openDoor(block, allowedToOpenSpecialDoor);
        if (upperBlock != undefined && typeof upperBlock != undefined) {
            openDoor(upperBlock, allowedToOpenSpecialDoor);
        }
    }
    else if (doorState === 16) {
        closeDoor(block, allowedToOpenSpecialDoor);
        if (upperBlock != undefined && typeof upperBlock != undefined) {
            closeDoor(upperBlock, allowedToOpenSpecialDoor);
        }
    }
    else if (block.hasTag('hcz_door') && doorState === 15) {
        closeDoor(block, allowedToOpenSpecialDoor);
        if (upperBlock != undefined && typeof upperBlock != undefined) {
            closeDoor(upperBlock, allowedToOpenSpecialDoor);
        }
    }
}
export function checking(dimension, startLoc, endLoc, allowedToOpenSpecialDoor) {
    const range = new CompoundBlockVolume();
    const volume = {
        action: CompoundBlockVolumeAction.Add,
        volume: new BlockVolume(startLoc, endLoc)
    };
    range.pushVolume(volume);
    for (let blockLoc of range.getBlockLocationIterator()) {
        const checkedBlock = dimension.getBlock(blockLoc);
        if (checkedBlock === undefined) {
            continue;
        }
        else {
            if (checkedBlock.hasTag('door:lower')) {
                interactWithDoor(checkedBlock, allowedToOpenSpecialDoor);
            }
        }
    }
    return false;
}
export function checkingForCpDoor(dimension, startLoc, endLoc) {
    const range = new CompoundBlockVolume();
    const substractLoc1 = {
        x: startLoc.x - 1,
        y: startLoc.y,
        z: startLoc.z - 1
    };
    const substractLoc2 = {
        x: endLoc.x + 1,
        y: endLoc.y,
        z: endLoc.z + 1
    };
    const addVolume = {
        action: CompoundBlockVolumeAction.Add,
        volume: new BlockVolume(startLoc, endLoc)
    };
    const substractVolume = {
        action: CompoundBlockVolumeAction.Subtract,
        volume: new BlockVolume(substractLoc1, substractLoc2)
    };
    range.pushVolume(addVolume);
    range.pushVolume(substractVolume);
    for (let blockLoc of range.getBlockLocationIterator()) {
        const checkedBlock = dimension.getBlock(blockLoc);
        if (checkedBlock === undefined) {
            continue;
        }
        else {
            if (checkedBlock.hasTag('door:lower')) {
                const perm = checkedBlock.permutation;
                const doorState = perm.getState('id:door');
                const upperBlock = checkedBlock.above(1);
                const upperPerm = upperBlock.permutation;
                if (doorState === 1) {
                    if (!checkedBlock.hasTag('cp_door')) {
                        return;
                    }
                    checkedBlock.setPermutation(perm.withState('id:door', 2).withState('is:open', false));
                    if (upperBlock != undefined && typeof upperBlock != undefined) {
                        if (!upperBlock.hasTag('cp_door')) {
                            return;
                        }
                        upperBlock.setPermutation(upperPerm.withState('id:door', 2).withState('is:open', false));
                    }
                }
                else if (doorState === 16) {
                    if (!checkedBlock.hasTag('cp_door')) {
                        return;
                    }
                    if (upperBlock != undefined && typeof upperBlock != undefined) {
                        checkedBlock.setPermutation(perm.withState('id:door', 15).withState('is:open', true));
                        if (!upperBlock.hasTag('cp_door')) {
                            return;
                        }
                        upperBlock.setPermutation(upperPerm.withState('id:door', 15).withState('is:open', true));
                    }
                }
            }
        }
    }
    return false;
}
export class button {
    onPlayerInteract(arg) {
        const dimension = arg.dimension;
        const block = arg.block;
        const player = arg.player;
        const perm = block.permutation;
        const state = perm.getState('fcb:state');
        const isi = perm.getState('is:interacted');
        if (state === undefined || isi === undefined || typeof state !== 'number' || typeof isi !== "boolean") {
            return;
        }
        const loc = arg.block.location;
        const locOfOnPlacedBlock = {
            x: loc.x,
            y: loc.y - 5,
            z: loc.z
        };
        const startLoc = {
            x: loc.x + 1,
            y: loc.y,
            z: loc.z + 1
        };
        const endLoc = {
            x: loc.x - 1,
            y: loc.y - 1,
            z: loc.z - 1
        };
        if (typeof player === 'undefined') {
            return;
        }
        const inventory = player.getComponent(EntityInventoryComponent.componentId);
        if (inventory === undefined) {
            return false;
        }
        const selectedItem = inventory.container?.getItem(player.selectedSlotIndex);
        if (selectedItem && selectedItem.typeId === "tool:change") {
            if (state === 1) {
                player.runCommand('title @s actionbar 方块状态已更新：锁定一');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 2) {
                player.runCommand('title @s actionbar 方块状态已更新：锁定二');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 3) {
                player.runCommand('title @s actionbar 方块状态已更新：未锁定');
                block.setPermutation(perm.withState('fcb:state', 1));
            }
        }
        //改变状态
        else {
            if (state === 1) {
                if (isi === false) {
                    block.dimension.playSound('random.click', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    checking(dimension, startLoc, endLoc, false);
                }
            }
            else if (state === 2) {
                block.dimension.playSound('button2', loc);
                player.runCommand('title @s actionbar 你按下了按钮，但是什么也没有发生');
            }
            else if (state === 3) {
                block.dimension.playSound('button2', loc);
                player.runCommand('title @s actionbar 你按下了按钮，但它似乎被锁定了');
            }
        }
        //触发事件
    }
}
//按钮触发组件
//电梯按钮触发组件
export class eleButton {
    onPlayerInteract(arg) {
        const dimension = arg.dimension;
        const block = arg.block;
        const player = arg.player;
        const perm = block.permutation;
        const state = perm.getState('fcb:state');
        const isi = perm.getState('is:interacted');
        if (state === undefined || isi === undefined || typeof state !== 'number' || typeof isi !== "boolean") {
            return;
        }
        const loc = arg.block.location;
        const locOfOnPlacedBlock = {
            x: loc.x,
            y: loc.y - 5,
            z: loc.z
        };
        if (typeof player === 'undefined') {
            return;
        }
        const inventory = player.getComponent(EntityInventoryComponent.componentId);
        if (inventory === undefined) {
            return false;
        }
        const selectedItem = inventory.container?.getItem(player.selectedSlotIndex);
        if (selectedItem && selectedItem.typeId === "tool:change") {
            if (state === 1) {
                player.runCommand('title @s actionbar 方块状态已更新：损坏');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 2) {
                player.runCommand('title @s actionbar 方块状态已更新：正常');
                block.setPermutation(perm.withState('fcb:state', 1));
            }
        }
        //改变状态
        else {
            if (state === 1) {
                if (isi === false) {
                    block.dimension.playSound('random.click', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                }
            }
            else if (state === 2) {
                block.dimension.playSound('button2', loc);
                player.runCommand('title @s actionbar 这个电梯好像损坏了');
            }
        }
        //触发事件
    }
}
export class reset {
    onTick(arg) {
        const block = arg.block;
        const dimension = arg.dimension;
        const perm = block.permutation;
        const state = perm.getState('fcb:state');
        const isi = perm.getState('is:interacted');
        if (state === undefined || isi === undefined || isi == false || typeof state !== 'number' || typeof isi !== "boolean") {
            return;
        }
        const loc = block.location;
        const locOfOnPlacedBlock = {
            x: loc.x,
            y: loc.y - 5,
            z: loc.z
        };
        if (isi === true) {
            block.setPermutation(perm.withState('is:interacted', false));
            dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:air');
        }
    }
}
//按钮，读卡器复位组件
export class keyreader {
    onPlayerInteract(arg) {
        const block = arg.block;
        const player = arg.player;
        const perm = block.permutation;
        const state = perm.getState('fcb:state');
        const isi = perm.getState('is:interacted');
        if (state === undefined || isi === undefined || typeof state !== 'number' || typeof isi !== "boolean") {
            return;
        }
        const loc = arg.block.location;
        const locOfOnPlacedBlock = {
            x: loc.x,
            y: loc.y - 5,
            z: loc.z
        };
        const startLoc = {
            x: loc.x + 1,
            y: loc.y,
            z: loc.z + 1
        };
        const endLoc = {
            x: loc.x - 1,
            y: loc.y - 1,
            z: loc.z - 1
        };
        const startLoc2 = {
            x: loc.x + 2,
            y: loc.y,
            z: loc.z + 2
        };
        const endLoc2 = {
            x: loc.x - 2,
            y: loc.y - 1,
            z: loc.z - 2
        };
        if (typeof player === 'undefined') {
            return;
        }
        const inventory = player.getComponent(EntityInventoryComponent.componentId);
        if (inventory === undefined) {
            return false;
        }
        const selectedItem = inventory.container?.getItem(player.selectedSlotIndex);
        if (selectedItem && selectedItem.typeId === "tool:change") {
            if (state === 0) {
                player.runCommand('title @s actionbar 方块状态已更新：lv1');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 1) {
                player.runCommand('title @s actionbar 方块状态已更新：lv2');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 2) {
                player.runCommand('title @s actionbar 方块状态已更新：lv3');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 3) {
                player.runCommand('title @s actionbar 方块状态已更新：lv4');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 4) {
                player.runCommand('title @s actionbar 方块状态已更新：lv5');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 5) {
                player.runCommand('title @s actionbar 方块状态已更新：锁定一');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 6) {
                player.runCommand('title @s actionbar 方块状态已更新：锁定二');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 7) {
                player.runCommand('title @s actionbar 方块状态已更新：lv0');
                block.setPermutation(perm.withState('fcb:state', 0));
            }
        } //改变状态
        else {
            if (state === 0 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level0") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv0
            else if (state === 1 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else if (selectedItem && selectedItem.typeId === "key:level0") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你需要一张一级或更高等级的权限卡');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv1
            else if (state === 2 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else if ((selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level0")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你需要一张二级或更高等级的权限卡');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv2
            else if (state === 3 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else if ((selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level0")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你需要一张三级或更高等级的权限卡');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv3
            else if (state === 4 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else if ((selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level0")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你需要一张四级或更高等级的权限卡');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv4
            else if (state === 5 && isi === false) {
                if ((selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key1', loc);
                    checking(block.dimension, startLoc, endLoc, true);
                    checkingForCpDoor(block.dimension, startLoc2, endLoc2);
                    block.setPermutation(perm.withState('is:interacted', true));
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把卡插入了卡槽');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else if ((selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level0")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你需要一张五级或更高等级的权限卡');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //lv5
            else if (state === 6) {
                if ((selectedItem && selectedItem.typeId === "key:level0") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽,但什么也没有发生');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //锁定一
            else if (state === 7) {
                if ((selectedItem && selectedItem.typeId === "key:level0") ||
                    (selectedItem && selectedItem.typeId === "key:level1") ||
                    (selectedItem && selectedItem.typeId === "key:level2") ||
                    (selectedItem && selectedItem.typeId === "key:level3") ||
                    (selectedItem && selectedItem.typeId === "key:level4") ||
                    (selectedItem && selectedItem.typeId === "key:level5") ||
                    (selectedItem && selectedItem.typeId === "key:omni")) {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽,但它似乎被锁定了');
                }
                else if (selectedItem && selectedItem.typeId === "key:level6") {
                    block.dimension.playSound('key2', loc);
                    player.runCommand('title @s actionbar 你把卡插入了卡槽。读卡器显示：“未知错误！黑月是否嚎叫？是，否。是，否......”');
                }
                else {
                    block.dimension.playSound('button2', loc);
                    player.runCommand('title @s actionbar 你需要一张权限卡来通过访问');
                }
            }
            //锁定二
        }
    }
}
//读卡器触发组件
export class DNAScanner {
    onPlayerInteract(arg) {
        const block = arg.block;
        const player = arg.player;
        const perm = block.permutation;
        const state = perm.getState('fcb:state');
        const isi = perm.getState('is:interacted');
        if (state === undefined || isi === undefined || typeof state !== 'number' || typeof isi !== "boolean") {
            return;
        }
        const loc = arg.block.location;
        const locOfOnPlacedBlock = {
            x: loc.x,
            y: loc.y - 5,
            z: loc.z
        };
        const startLoc = {
            x: loc.x + 1,
            y: loc.y,
            z: loc.z + 1
        };
        const endLoc = {
            x: loc.x - 1,
            y: loc.y - 1,
            z: loc.z - 1
        };
        // const startLoc2 ={
        //     x:loc.x+2,
        //     y:loc.y,
        //     z:loc.z+2
        // }
        // const endLoc2 ={
        //     x:loc.x-2,
        //     y:loc.y-1,
        //     z:loc.z-2
        // }
        if (typeof player === 'undefined') {
            return;
        }
        const inventory = player.getComponent(EntityInventoryComponent.componentId);
        if (inventory === undefined) {
            return false;
        }
        const selectedItem = inventory.container?.getItem(player.selectedSlotIndex);
        if (selectedItem && selectedItem.typeId === "tool:change") {
            if (state === 1) {
                player.runCommand('title @s actionbar 方块状态已更新:Black');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 2) {
                player.runCommand('title @s actionbar 方块状态已更新：Error');
                block.setPermutation(perm.withState('fcb:state', state + 1));
            }
            else if (state === 3) {
                player.runCommand('title @s actionbar 方块状态已更新：White');
                block.setPermutation(perm.withState('fcb:state', 1));
            }
        } //改变状态
        else {
            if (state === 1) {
                if (selectedItem && selectedItem.typeId === "key:severed_hand1") {
                    block.dimension.playSound('check1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, false);
                    // checkingForCpDoor(block.dimension,startLoc2,endLoc2)
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把手掌放在了扫描仪上，扫描仪显示：“DNA类型匹配，允许访问”');
                }
                else {
                    block.dimension.playSound('check2', loc);
                    player.runCommand('title @s actionbar 你把手掌放在了扫描仪上，扫描仪显示：“未知的DNA类型，拒绝访问”');
                }
            }
            else if (state === 2) {
                if (selectedItem && selectedItem.typeId === "key:severed_hand2") {
                    block.dimension.playSound('check1', loc);
                    block.setPermutation(perm.withState('is:interacted', true));
                    checking(block.dimension, startLoc, endLoc, true);
                    // checkingForCpDoor(block.dimension,startLoc2,endLoc2)
                    arg.dimension.setBlockType(locOfOnPlacedBlock, 'minecraft:redstone_block');
                    player.runCommand('title @s actionbar 你把手掌放在了扫描仪上，扫描仪显示：“DNA类型匹配，允许访问”');
                }
                else {
                    block.dimension.playSound('check2', loc);
                    player.runCommand('title @s actionbar 你把手掌放在了扫描仪上，扫描仪显示：“未知的DNA类型，拒绝访问”');
                }
            }
            else if (state === 3) {
                block.dimension.playSound('check2', loc);
                player.runCommand('title @s actionbar 你把手掌放在了扫描仪上，但是什么也没有发生。扫描仪显示：“错误！无法连接到DNA类型数据库！”');
            }
        }
    }
}
//DNA扫描仪触发组件
