import { system } from "@minecraft/server";
//门的开关组件
export class changeState {
    onTick(arg) {
        const block = arg.block;
        const blockID = block.typeId;
        const perm = block.permutation;
        const state = perm.getState('id:door');
        const isOpen = perm.getState('is:open');
        let sound = {
            opening: 'ldo',
            closing: 'ldc'
        };
        if (block.hasTag('hcz_door')) {
            sound = { opening: "hdo", closing: 'hdc' };
        }
        else if (block.hasTag('ele_door')) {
            sound = { opening: "edo", closing: 'edc' };
        }
        else if (block.hasTag('cp_door')) {
            sound = { opening: "cpd", closing: 'cpd' };
        }
        const isUpper = perm.getState('is:upper');
        if (state === undefined || typeof state !== 'number' ||
            isUpper === undefined || typeof isUpper === 'undefined') {
            return;
        }
        const loc = arg.block.location;
        if (blockID !== 'hcz:door') {
            if (isOpen === false && state < 16) {
                block.setPermutation(perm.withState('id:door', state + 1));
                if (state == 2 && isUpper === false) {
                    block.dimension.playSound(sound.opening, loc);
                }
            }
            else if (isOpen === true && state > 1) {
                block.setPermutation(perm.withState('id:door', state - 1));
                if (state == 15 && isUpper === false) {
                    block.dimension.playSound(sound.closing, loc);
                    if (block.hasTag('cp_door')) {
                        block.dimension.runCommand('execute positioned ' + loc.x + ' ' + loc.y + ' ' + loc.z + ' ' + ' run stopsound @a[r=15] door_beep');
                    }
                }
            }
        }
        else if (blockID === 'hcz:door') {
            if (isOpen === false && state < 15) {
                block.setPermutation(perm.withState('id:door', state + 1));
                if (state == 2 && isUpper === false) {
                    block.dimension.playSound(sound.opening, loc);
                }
            }
            else if (isOpen === true && state > 1) {
                block.setPermutation(perm.withState('id:door', state - 1));
                if (state == 14 && isUpper === false) {
                    block.dimension.playSound(sound.closing, loc);
                }
            }
        }
    }
}
//门的自动补全组件
export class complete {
    onPlace(arg) {
        const block = arg.block;
        const blockID = block.typeId;
        const perm = block.permutation;
        const lowerBlock = block.below(1);
        const direction = perm.getState('minecraft:cardinal_direction');
        if (direction === undefined || typeof direction === 'undefined') {
            return;
        }
        const onPlacedBlockLoc = {
            x: block.location.x,
            y: block.location.y + 1,
            z: block.location.z
        };
        if (lowerBlock === undefined || typeof lowerBlock === "undefined") {
            return;
        }
        if (lowerBlock.hasTag('door:lower')) {
            block.setPermutation(perm.withState('is:upper', true));
        }
        else {
            block.dimension.setBlockType(onPlacedBlockLoc, blockID);
            block.dimension.setBlockPermutation(onPlacedBlockLoc, perm.withState('minecraft:cardinal_direction', direction));
        }
    }
}
//检查点门自动关门
export class cpDoor {
    onTick(arg) {
        const block = arg.block;
        const perm = block.permutation;
        const loc = block.location;
        const state = perm.getState('id:door');
        const isOpen = perm.getState('is:open');
        const isUpper = perm.getState('is:upper');
        const check = perm.getState('id:check');
        if (state === undefined || typeof state !== 'number' ||
            isOpen === undefined || typeof isOpen === 'undefined' ||
            isUpper === undefined || typeof isUpper === 'undefined') {
            return;
        }
        if (check === false) {
            block.setPermutation(perm.withState('id:check', true));
        }
        else {
            if (isUpper === false) {
                block.dimension.playSound('door_beep', loc);
            } //播放音频
            system.runTimeout(() => {
                if (block.permutation.getState('id:door') === 16) {
                    block.setPermutation(perm.withState('id:door', 15).withState('is:open', true));
                } //2s后关门
            }, 40);
        }
    }
}
//其他门的自动关门
export class autoClose {
    onTick(arg) {
        const block = arg.block;
        const perm = block.permutation;
        const loc = block.location;
        const state = perm.getState('id:door');
        const isOpen = perm.getState('is:open');
        const isUpper = perm.getState('is:upper');
        const check = perm.getState('id:check');
        if (state === undefined || typeof state !== 'number' ||
            isOpen === undefined || typeof isOpen === 'undefined' ||
            isUpper === undefined || typeof isUpper === 'undefined') {
            return;
        }
        if (check === false) {
            block.setPermutation(perm.withState('id:check', true));
        }
        else {
            if (isUpper === false) {
                block.dimension.playSound('door_beep', loc);
            } //播放音频
            system.runTimeout(() => {
                if (block.permutation.getState('id:door') === 16) {
                    block.setPermutation(perm.withState('id:door', 15).withState('is:open', true).withState('is:mtf', false));
                }
                else if (block.permutation.getState('id:door') === 15 && block.hasTag('hcz_door')) {
                    block.setPermutation(perm.withState('id:door', 14).withState('is:open', true).withState('is:mtf', false));
                    return;
                } //2s后关门
            }, 40);
        }
    }
}
