import { system, world, CompoundBlockVolume, CompoundBlockVolumeAction, BlockVolume, Block, WorldAfterEvents } from "@minecraft/server";
import { button, keyreader, reset, DNAScanner, eleButton } from "./button";
import { changeState, complete, cpDoor, autoClose} from "./door";
import { passcodeButton, pcReset } from "./passcodeButton";
import { signalBlock } from "./signalBlock";

world.setDynamicProperty('remoteDoorControl',true)
let hasNoticed = false

//方块自定义组件注册
{
    world.beforeEvents.worldInitialize.subscribe(function (initEvent) {
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:button", new button());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:eleButton", new eleButton());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:reset", new reset());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:keyreader", new keyreader());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:DNAScanner", new DNAScanner());
        initEvent.blockComponentRegistry.registerCustomComponent("door:changeState", new changeState());
        initEvent.blockComponentRegistry.registerCustomComponent("door:complete", new complete());
        initEvent.blockComponentRegistry.registerCustomComponent("signal:block", new signalBlock());
        initEvent.blockComponentRegistry.registerCustomComponent("door:cpDoor", new cpDoor());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:passcodeButton", new passcodeButton());
        initEvent.blockComponentRegistry.registerCustomComponent("fcb:pcReset", new pcReset());
        initEvent.blockComponentRegistry.registerCustomComponent("door:autoClose", new autoClose());
    });
}

//add控制指令
{
    world.afterEvents.chatSend.subscribe((eventData) => {
        const message = eventData.message
        if (!message.includes('>')) { return }
        const args = message.split(' ')
        if (args.length > 0) {
            switch (args[0]) {
                case '>show': show()
                    break;
                case '>set': set(args)
                    break;
                case '>help': world.sendMessage('§a-可用的指令<1/1>：\n >show 查看add全局变量 \n >set 设置add全局变量，后接变量名 \n >fox 呼叫fox(?)')
                    break;
                case '>fox': fox()
                    break;
                default: world.sendMessage('§c 未知的指令！请输入>help查看帮助')
            }
        }
    })

    function show() {
        world.sendMessage('§a 远程门控是否启用：' + world.getDynamicProperty('remoteDoorControl'))
    }
    function set(args) {
        switch (args[1]) {
            case 'mtf': {
                if (args[2] == 'true') {
                    world.setDynamicProperty('remoteDoorControl',true)
                    world.sendMessage('§a 设置成功，现在mtf可以自动开门')
                }
                else if (args[2] == 'false') {
                    world.setDynamicProperty('remoteDoorControl',false)
                    world.sendMessage('§a 设置成功，现在mtf不可以自动开门')
                }
                else if (args.length != 3) {
                    world.sendMessage('§c 错误！接受的格式：>set mtf true/false')
                }
                else {
                    world.sendMessage('§c 错误！接受的格式：>set mtf true/false')
                }
            } break;
        }
    }
    function fox() {
        system.waitTicks(10)
            .then(() => { world.sendMessage("§eFoxTrot-Unit1 加入了游戏") })
            .then(() => {
                system.waitTicks(40)
                    .then(() => { world.sendMessage("<FoxTrot-Unit1> 神金") })
                    .then(() => {
                        system.waitTicks(80)
                            .then(() => { world.sendMessage("<FoxTrot-Unit1> byd没事叫我干啥，阿弥诺斯，下了 :(") })
                            .then(() => {
                                system.waitTicks(20)
                                    .then(() => { world.sendMessage("§eFoxTrot-Unit1 退出了游戏") })

                            })
                    })

            })
    }
}

//神金小提示awa
{
    const notice = () => {
        if (hasNoticed == true) { return }
        hasNoticed = true
        system.waitTicks(1200)
            .then(() => { world.sendMessage("§eFoxTrot-Unit1 加入了游戏") })
            .then(() => {
                system.waitTicks(300)
                    .then(() => { world.sendMessage("<FoxTrot-Unit1> 你说的对,但是CB Doors v2 add 是由b站up主FoxTrot-Unit1开发的一款mc附加包,由鸭梨(b站up主屑建筑九尾)提供绝大多数模型。一定要去看看下载add时附带的须知＆帮助.txt！") })
                    .then(() => {
                        system.waitTicks(80)
                            .then(() => { world.sendMessage("<FoxTrot-Unit1> 我滴任务完成啦，呵呵，啊哈哈哈哈") })
                            .then(() => {
                                system.waitTicks(20)
                                    .then(() => { world.sendMessage("§eFoxTrot-Unit1 退出了游戏") })

                            })
                    })

            })
    }//作者声明

    world.afterEvents.playerJoin.subscribe(notice)
}

//mtf 自动开门
{
    function openDoor(block, allowedToOpenSpecialDoor) {
        const perm = block.permutation
        if (allowedToOpenSpecialDoor === false && !block.hasTag('door:common')) { return }
        block.setPermutation(perm.withState('id:door', 2).withState('is:open', false).withState('is:mtf',true))
    }//开门

    function interactWithDoor(block, allowedToOpenSpecialDoor, mtfType) {
        const perm = block.permutation
        const doorState = perm.getState('id:door')
        const upperBlock = block.above(1)
        if (doorState === 1) {
            if (mtfType == 'mtf') { block.dimension.playSound('door_mtf', block.location) }
            else if (mtfType == 'sne') { block.dimension.playSound('door_sne', block.location) }
            openDoor(block, allowedToOpenSpecialDoor)
            if (upperBlock != undefined && typeof upperBlock != undefined) {
                openDoor(upperBlock, allowedToOpenSpecialDoor)
            }
        }
    }//与门互动

    function checking(dimension, startLoc, endLoc, allowedToOpenSpecialDoor, mtfType) {
        const range = new CompoundBlockVolume()
        const volume = {
            action: CompoundBlockVolumeAction.Add,
            volume: new BlockVolume(startLoc, endLoc)
        }
        if(!world.getDynamicProperty('remoteDoorControl')){return}
        range.pushVolume(volume)
        for (let blockLoc of range.getBlockLocationIterator()) {
            const checkedBlock = dimension.getBlock(blockLoc)
            if (checkedBlock === undefined) {
                continue
            }
            else {
                if (checkedBlock.hasTag('door:lower')) {
                    if(checkedBlock.hasTag('ele_door')){return}
                    interactWithDoor(checkedBlock, allowedToOpenSpecialDoor, mtfType)
                }
            }
        }
        return false
    }//mtf搜寻门
    const overWorld = world.getDimension('overworld')
    const nether = world.getDimension('nether')
    const theEnd = world.getDimension('the_end')
    let mtfOpenDoors = () => {
        let entities = overWorld.getEntities().concat(nether.getEntities(), theEnd.getEntities())
        let mtfs = []
        let snes = []
        for (const entity of entities) {
            if (entity.hasTag('mtf') && !entity.hasTag('sne')) { mtfs.push(entity) }
            else if (entity.hasTag('sne')) (snes.push(entity))
        }
        for (const mtf of mtfs) {
            const startLoc = {
                x: mtf.location.x - 2,
                y: mtf.location.y - 1,
                z: mtf.location.z - 2,
            }
            const endLoc = {
                x: mtf.location.x + 2,
                y: mtf.location.y + 1,
                z: mtf.location.z + 2,
            }
            checking(mtf.dimension, startLoc, endLoc, true, 'mtf')
        }
        for (const sne of snes) {
            const startLoc = {
                x: sne.location.x - 2,
                y: sne.location.y - 1,
                z: sne.location.z - 2,
            }
            const endLoc = {
                x: sne.location.x + 2,
                y: sne.location.y + 1,
                z: sne.location.z + 2,
            }
            checking(sne.dimension, startLoc, endLoc, true, 'sne')
        }
    }
    system.runInterval(mtfOpenDoors, 5)
}

//给MCB里的mtf自动添加tag
{
    world.afterEvents.entitySpawn.subscribe((eventData)=>{
        const entity = eventData.entity
        if(entity.typeId == 'bf:cb_ntf' || entity.typeId == 'bf:cb_ntf_commander' || eventData.entity.typeId == 'bf:cb_ntf_medic' || eventData.entity.typeId == 'bf:cb_d7'){
            entity.addTag('mtf')
        }
        else if(entity.typeId == 'bf:cb_sne' ){
            entity.addTag('sne')
        }
    })
}

//远控权限非玩家聊天框指令改变
{
    system.afterEvents.scriptEventReceive.subscribe((eventData)=>{
        const id = eventData.id
        const message = eventData.message
        switch(id.toLocaleLowerCase){
            case 'fcb:set':set(message);break
        }
        function set(message){
            switch(message.toLocaleLowerCase){
                case 'doorremotecontrol':{
                    if(world.getDynamicProperty('remoteDoorControl')){
                        world.setDynamicProperty('remoteDoorControl',false)
                    }
                    else {world.setDynamicProperty('remoteDoorControl',true)}
                };break
            }
        }
    })
}